@@ -65,7 +65,7 @@ for my $line (@raw_lines) {
 my $current_state;
 
 for my $line (@lines) {
-    if (my ($state) = ($line =~ /^state ([A-Z0-9_]+):$/)) {
+    if (my ($state) = ($line =~ /^state ([A-Z_]+):$/)) {
         #say "got a new state: $state";
         $current_state = $state;
     } else {
@@ -77,7 +77,7 @@ for my $line (@lines) {
             ($line =~ /
                 ^\s*                  # skip leading whitespace
                 ([a-z_]+ \s* = \s*|)  # optional identifier
-                (.*?) -> \s*          # token
+                (.*?) -> \s*          # token 
                 (.*)                  # optional action
              /x);
 
@@ -109,23 +109,24 @@ for my $line (@lines) {
 # Second step: Generate the enum values for all states.
 
 # It is important to keep the order the same, so we store the keys once.
-# We sort descendingly by length to be able to replace occurrences of the state
+# We sort descendingly by length to be able to replace occurences of the state
 # name even when one state’s name is included in another one’s (like FOR_WINDOW
 # is in FOR_WINDOW_COMMAND).
 my @keys = sort { (length($b) <=> length($a)) or ($a cmp $b) } keys %states;
 
 open(my $enumfh, '>', "GENERATED_${prefix}_enums.h");
 
+# XXX: we might want to have a way to do this without a trailing comma, but gcc
+# seems to eat it.
 my %statenum;
 say $enumfh 'typedef enum {';
 my $cnt = 0;
 for my $state (@keys, '__CALL') {
-    say $enumfh ',' if $cnt > 0;
-    print $enumfh "    $state = $cnt";
+    say $enumfh "    $state = $cnt,";
     $statenum{$state} = $cnt;
     $cnt++;
 }
-say $enumfh "\n} cmdp_state;";
+say $enumfh '} cmdp_state;';
 close($enumfh);
 
 # Third step: Generate the call function.
@@ -154,20 +155,12 @@ for my $state (@keys) {
         # to generate a format string. The format uses %d for <number>s,
         # literal numbers or state IDs and %s for NULL, <string>s and literal
         # strings.
-
-        # remove the function name temporarily, so that the following
-        # replacements only apply to the arguments.
-        my ($funcname) = ($fmt =~ /^(.+)\(/);
-        $fmt =~ s/^$funcname//;
-
         $fmt =~ s/$_/%d/g for @keys;
         $fmt =~ s/\$([a-z_]+)/%s/g;
         $fmt =~ s/\&([a-z_]+)/%ld/g;
         $fmt =~ s/"([a-z0-9_]+)"/%s/g;
         $fmt =~ s/(?:-?|\b)[0-9]+\b/%d/g;
 
-        $fmt = $funcname . $fmt;
-
         say $callfh "         case $call_id:";
         say $callfh "             result->next_state = $next_state;";
         say $callfh '#ifndef TEST_PARSER';
@@ -224,7 +217,7 @@ for my $state (@keys) {
             $next_state = '__CALL';
         }
         my $identifier = $token->{identifier};
-        say $tokfh qq|    { "$token_name", "$identifier", $next_state, { $call_identifier } },|;
+        say $tokfh qq|    { "$token_name", "$identifier", $next_state, { $call_identifier } }, |;
     }
     say $tokfh '};';
 }
