@@ -2,13 +2,12 @@
  * vim:ts=4:sw=4:expandtab
  *
  * i3 - an improved dynamic tiling window manager
- * © 2009-2011 Michael Stapelberg and contributors (see also: LICENSE)
+ * © 2009 Michael Stapelberg and contributors (see also: LICENSE)
  *
  * click.c: Button press (mouse click) events.
  *
  */
-#ifndef I3_CLICK_H
-#define I3_CLICK_H
+#pragma once
 
 /**
  * The button press X callback. This function determines whether the floating
@@ -19,5 +18,3 @@
  *
  */
 int handle_button_press(xcb_button_press_event_t *event);
-
-#endif
